@@ -2,6 +2,9 @@
 // Copyright (c) 2022-2023 Sophie Herold
 // Copyright (c) 2022 Elton A Rodrigues
 // Copyright (c) 2022 Maximiliano Sandoval R
+// Copyright (c) 2023 FineFindus
+// Copyright (c) 2023 qwel
+// Copyright (c) 2023 Huan Thieu Nguyen
 // Copyright (c) 2023 Sabri Ünal
 // Copyright (c) 2023 Lubosz Sarnecki
 //
@@ -29,14 +32,12 @@ use adw::subclass::prelude::*;
 use glib::clone;
 use gtk::CompositeTemplate;
 
-use once_cell::sync::OnceCell;
-
-use std::cell::{Cell, RefCell};
+use std::cell::{Cell, OnceCell, RefCell};
 use std::path::{Path, PathBuf};
 
 use crate::config;
 use crate::util::{self, Direction, Position};
-use crate::widgets::{LpImage, LpImageView, LpPropertiesView};
+use crate::widgets::{LpDragOverlay, LpImage, LpImageView, LpPropertiesView};
 
 /// Show window after X milliseconds even if image dimensions are not known yet
 const SHOW_WINDOW_AFTER: u64 = 2000;
@@ -89,8 +90,17 @@ mod imp {
         pub(super) image_view: TemplateChild<LpImageView>,
         #[template_child]
         pub(super) properties_view: TemplateChild<LpPropertiesView>,
+
+        #[template_child]
+        pub(super) drag_overlay: TemplateChild<LpDragOverlay>,
         #[template_child]
         pub(super) drop_target: TemplateChild<gtk::DropTarget>,
+
+        #[template_child]
+        pub(super) forward_click_gesture: TemplateChild<gtk::GestureClick>,
+        #[template_child]
+        pub(super) backward_click_gesture: TemplateChild<gtk::GestureClick>,
+
         /// Motion controller for complete window
         pub(super) motion_controller: gtk::EventControllerMotion,
         pub(super) pointer_position: Cell<(f64, f64)>,
@@ -241,6 +251,15 @@ mod imp {
 
             self.parent_constructed();
 
+            self.forward_click_gesture
+                .connect_pressed(clone!(@weak obj => move |_,_,_,_| {
+                    obj.image_view().navigate(Direction::Forward);
+                }));
+            self.backward_click_gesture
+                .connect_pressed(clone!(@weak obj => move |_,_,_,_| {
+                    obj.image_view().navigate(Direction::Back);
+                }));
+
             if config::PROFILE == ".Devel" {
                 obj.add_css_class("devel");
             }
@@ -360,12 +379,24 @@ mod imp {
 
             self.drop_target.set_types(&[gdk::FileList::static_type()]);
 
+            self.drop_target.connect_accept(
+                clone!(@weak obj => @default-return false, move |_drop_target, drop| {
+                    // Only accept drops from external sources or different windows
+                    let different_source = drop.drag().is_none() || drop.drag() != obj.image_view().drag_source().drag();
+                    // We have to do this manually since we are overwriting the default handler
+                    let correct_format = drop.formats().contains_type(gdk::FileList::static_type());
+
+                    different_source && correct_format
+                }),
+            );
+
             // For callbacks, you will want to reference the GTK docs on
             // the relevant signal to see which parameters you need.
             // In this case, we need only need the GValue,
             // so we name it `value` then use `_` for the other spots.
             self.drop_target.connect_drop(
                 clone!(@weak obj => @default-return false, move |_, value, _, _| {
+
                     // Here we use a GValue, which is a dynamic object that can hold different types,
                     // e.g. strings, numbers, or in this case objects. In order to get the GdkFileList
                     // from the GValue, we need to use the `get()` method.
@@ -373,7 +404,7 @@ mod imp {
                     // We've added type annotations here, and written it as `let list: gdk::FileList = ...`,
                     // but you might also see places where type arguments are used.
                     // This line could have been written as `let list = value.get::<gdk::FileList>().unwrap()`.
-                    let mut files = match value.get::<gdk::FileList>() {
+                    let files = match value.get::<gdk::FileList>() {
                         Ok(list) => list.files(),
                         Err(err) => {
                             log::error!("Issue with drop value: {err}");
@@ -381,10 +412,8 @@ mod imp {
                         }
                     };
 
-                    if files.len() > 1 {
+                    if !files.is_empty() {
                         obj.image_view().set_images_from_files(files);
-                    } else if let Some(file) = files.pop() {
-                        obj.image_view().set_image_from_file(file);
                     } else {
                         log::error!("Dropped FileList was empty");
                         return false;
@@ -448,13 +477,16 @@ impl LpWindow {
     }
 
     async fn pick_file(&self) {
-        let filter_list = gio::ListStore::new(gtk::FileFilter::static_type());
+        let filter_list = gio::ListStore::new::<gtk::FileFilter>();
 
         let filter_supported_formats = gtk::FileFilter::new();
         filter_supported_formats.set_name(Some(&gettext("Supported image formats")));
-        filter_supported_formats.add_mime_type("image/*");
-        // Manual support for Farbfeld since it has no mime type
-        filter_supported_formats.add_suffix("ff");
+        for mime_type in glycin::image_formats().await {
+            filter_supported_formats.add_mime_type(&mime_type);
+        }
+        // TODO: Update when SVG moves to glycin as well
+        filter_supported_formats.add_mime_type("image/svg+xml");
+        filter_supported_formats.add_mime_type("image/svg+xml-compressed");
 
         let filter_all_files = gtk::FileFilter::new();
         filter_all_files.set_name(Some(&gettext("All files")));
@@ -470,8 +502,16 @@ impl LpWindow {
             .modal(true)
             .build();
 
-        if let Ok(file) = chooser.open_future(Some(self)).await {
-            self.image_view().set_image_from_file(file);
+        if let Ok(selected) = chooser.open_multiple_future(Some(self)).await {
+            let images: Vec<_> = selected
+                .into_iter()
+                .filter_map(|files| {
+                    files
+                        .ok()
+                        .and_then(|object| object.downcast::<gio::File>().ok())
+                })
+                .collect();
+            self.image_view().set_images_from_files(images);
         } else {
             log::debug!("File dialog canceled or file not readable");
         }
@@ -482,6 +522,7 @@ impl LpWindow {
 
         if let Some(ref file) = imp.image_view.current_file() {
             let launcher = gtk::FileLauncher::new(Some(file));
+            launcher.set_always_ask(true);
             if let Err(e) = launcher.launch_future(Some(self)).await {
                 if !e.matches(gtk::DialogError::Dismissed) {
                     log::error!("Could not open image in external program: {}", e);
@@ -527,8 +568,13 @@ impl LpWindow {
 
     async fn trash(&self) {
         let image_view = self.image_view();
-        let (Some(file), Some(path)) = (image_view.current_file(), image_view.current_file().and_then(|x| x.path()))
-            else { log::error!("No file to trash"); return; };
+        let (Some(file), Some(path)) = (
+            image_view.current_file(),
+            image_view.current_file().and_then(|x| x.path()),
+        ) else {
+            log::error!("No file to trash");
+            return;
+        };
 
         let result = file.trash_future(glib::Priority::default()).await;
 
@@ -537,13 +583,14 @@ impl LpWindow {
                 let toast = adw::Toast::builder()
                     .title(gettext("Image moved to trash"))
                     .button_label(gettext("Undo"))
+                    .priority(adw::ToastPriority::High)
                     .build();
                 toast.connect_button_clicked(glib::clone!(@weak self as win => move |_| {
                     let path = path.clone();
                     spawn(async move {
                         let result = crate::util::untrash(&path).await;
                         match result {
-                            Ok(()) => win.image_view().set_image_from_file(gio::File::for_path(&path)),
+                            Ok(()) => win.image_view().set_images_from_files(vec![gio::File::for_path(&path)]),
                             Err(err) => {
                                 log::error!("Failed to untrash {path:?}: {err}");
                                 win.show_toast(
@@ -631,7 +678,6 @@ impl LpWindow {
         self.action_set_enabled("win.rotate_cw", enabled);
         self.action_set_enabled("win.rotate_ccw", enabled);
         self.action_set_enabled("win.copy", enabled);
-        self.action_set_enabled("win.trash", enabled);
         self.action_set_enabled("win.zoom-best-fit", enabled);
         self.action_set_enabled("win.zoom-to-exact", enabled);
         self.action_set_enabled("win.toggle-properties", enabled);
@@ -645,6 +691,12 @@ impl LpWindow {
 
         imp.properties_button.set_sensitive(shows_image);
         self.set_actions_enabled(shows_image);
+        self.action_set_enabled(
+            "win.trash",
+            imp.image_view
+                .current_file()
+                .is_some_and(|file| file.path().is_some()),
+        );
 
         if shows_image {
             imp.stack.set_visible_child(&*imp.image_view);
@@ -670,13 +722,11 @@ impl LpWindow {
             .current_page()
             .map(|page| page.image());
 
-        if let Some(image) = image {
-            if image.image_size() > (0, 0) {
-                log::debug!("Showing window because image size is ready");
-                // this let's the window determine the default size from LpImage's natural size
-                self.set_default_size(-1, -1);
-                self.present();
-            }
+        if image.is_some_and(|img| img.image_size() > (0, 0)) {
+            log::debug!("Showing window because image size is ready");
+            // this let's the window determine the default size from LpImage's natural size
+            self.set_default_size(-1, -1);
+            self.present();
         }
     }
 
@@ -687,11 +737,9 @@ impl LpWindow {
 
         let current_page = self.imp().image_view.current_page();
 
-        if let Some(page) = current_page {
-            if page.image().error().is_some() {
-                log::debug!("Showing window because loading image failed");
-                self.present();
-            }
+        if current_page.is_some_and(|page| page.image().error().is_some()) {
+            log::debug!("Showing window because loading image failed");
+            self.present();
         }
     }
 
@@ -832,13 +880,13 @@ impl LpWindow {
     fn window_title(&self, file: Option<&gio::File>) -> String {
         // ensure that templates are initialized
         if file.is_none() {
-            gettext("Loupe")
+            gettext("Image Viewer")
         } else {
             self.imp()
                 .image_view
                 .current_file()
                 .and_then(|f| util::get_file_display_name(&f)) // If the file exists, get display name
-                .unwrap_or_else(|| gettext("Loupe")) // Return that or the default if there's nothing
+                .unwrap_or_else(|| gettext("Image Viewer")) // Return that or the default if there's nothing
         }
     }
 
@@ -922,11 +970,14 @@ impl LpWindow {
         fullscreened: bool,
         show_properties: bool,
         show_menu: bool,
+        current_image: Option<LpImage>,
     ) -> adw::ToolbarStyle {
-        if self.extend_to_top(fullscreened, show_properties, show_menu) {
+        // Flat headerbar for empty state and fullscreen
+        if current_image.is_none() || self.extend_to_top(fullscreened, show_properties, show_menu) {
             adw::ToolbarStyle::Flat
         } else {
-            adw::ToolbarStyle::Raised
+            // Use the border variant of raised to avoid shadows over images
+            adw::ToolbarStyle::__Unknown(2)
         }
     }
 }
